@@ -11,6 +11,7 @@ Date: May 29, 2021
 
 import collections
 import math
+import warnings
 
 import geopandas as gpd
 import numpy as np
@@ -20,7 +21,6 @@ from shapely.geometry import LineString, MultiLineString
 
 
 class COINS:
-
     """
     Calculates natural continuity and hierarchy of street networks in a given
     GeoDataFrame using the COINS algorithm.
@@ -38,10 +38,24 @@ class COINS:
     ----------
     edge_gdf : GeoDataFrame
         A GeoDataFrame containing edge geometry of a street network.
+        ``edge_gdf`` cannot contain identical or overlapping LineStrings.
         ``edge_gdf`` should ideally not contain MultiLineStrings.
-    angle_threshold : int, float (default 0)
-        The angle threshold for the COINS algorithm. Segments will only be considered
-        a part of the same street if the deflection angle is above the threshold.
+    angle_threshold : int, float (default 0), units: degrees
+        The threshold for the interior angle within the COINS algorithm.
+        Possible values: ``0 <= angle_threshold < 180``, in degrees.
+        Segments will only be considered part of the same stroke group
+        if the interior angle between them is above the threshold.
+    flow_mode : bool, default False
+        Continuity can be derived based on either visibility (``flow_mode=False``) or
+        flow (``flow_mode=True``). In the former case, a stroke group break is created
+        at any angle above the ``angle_threshold``, even at internal nodes within the
+        LineString (so one LineString can be divided into multiple stroke groups if its
+        segments connect at an angle above ``angle_threshold``). This corresponds to
+        visibility-based continuity. In the latter case, stroke group breaks are only
+        created at the end points of LineStrings, following the "flow" definition of
+        continuity where the direction of flow can change only at intersections. This
+        also ensures that each LineString can be assigned only a single stroke group.
+        Note that this option is not covered by :cite:`tripathy2020open`.
 
     Examples
     --------
@@ -62,10 +76,10 @@ class COINS:
     -----
     The LineStrings of the ``edge_gdf`` are not expected to overlap. If you are creating
     it using OSMnx, don't forget to cast the graph to undirected using
-    ``osmnx.get_undirected(G)`` prior converting it to a GeoDataFrame.
+    ``osmnx.convert.to_undirected(G)`` prior converting it to a GeoDataFrame.
     """
 
-    def __init__(self, edge_gdf, angle_threshold=0):
+    def __init__(self, edge_gdf, angle_threshold=0, flow_mode=False):
         self.edge_gdf = edge_gdf
         self.gdf_projection = self.edge_gdf.crs
         self.already_merged = False
@@ -89,7 +103,7 @@ class COINS:
         self._best_link()
 
         # cross check best links and enter angle threshold for connectivity
-        self._cross_check_links(angle_threshold)
+        self._cross_check_links(angle_threshold, flow_mode)
 
     def _premerge(self):
         """
@@ -99,18 +113,26 @@ class COINS:
         return self._create_gdf_premerge()
 
     def stroke_gdf(self):
-        """Return a GeoDataFrame containing merged final stroke geometry."""
+        """Return a GeoDataFrame containing merged final stroke geometry.
+
+        Returns
+        -------
+        GeoDataFrame
+        """
         if not self.already_merged:
             self._merge_lines()
         return self._create_gdf_strokes()
 
-    def stroke_attribute(self):
+    def stroke_attribute(self, return_ends=False):
         """
         Return a pandas Series encoding stroke groups onto the original input geometry.
+
+        Optionally, (``return_ends=True``), return a tuple of Series with the second
+        tuple encoding stroke group ends.
         """
         if not self.already_merged:
             self._merge_lines()
-        return self._add_gdf_stroke_attributes()
+        return self._add_gdf_stroke_attributes(return_ends=return_ends)
 
     def _split_lines(self):
         out_line = []
@@ -122,7 +144,7 @@ class COINS:
                 out_line.append(
                     [
                         part,
-                        _compute_orientation(part),
+                        [],
                         [],
                         [],
                         [],
@@ -215,15 +237,20 @@ class COINS:
             else:
                 self.unique[edge][5] = "dead_end"
 
-    def _cross_check_links(self, angle_threshold):
+    def _cross_check_links(self, angle_threshold, flow_mode):
         for edge in range(0, len(self.unique)):
             best_p1 = self.unique[edge][4][0]
             best_p2 = self.unique[edge][5][0]
 
             if (
-                isinstance(best_p1, int)
+                isinstance(best_p1, int)  # not dead_end
                 and edge in [self.unique[best_p1][4][0], self.unique[best_p1][5][0]]
                 and self.angle_pairs[f"{edge}_{best_p1}"] > angle_threshold
+            ) or (
+                flow_mode
+                and isinstance(best_p1, int)  # not dead_end
+                and edge in [self.unique[best_p1][4][0], self.unique[best_p1][5][0]]
+                and len(self.unique[edge][2]) == 1  # node degree 2
             ):
                 self.unique[edge][6] = best_p1
             else:
@@ -233,6 +260,11 @@ class COINS:
                 isinstance(best_p2, int)
                 and edge in [self.unique[best_p2][4][0], self.unique[best_p2][5][0]]
                 and self.angle_pairs[f"{edge}_{best_p2}"] > angle_threshold
+            ) or (
+                flow_mode
+                and isinstance(best_p2, int)  # not dead_end
+                and edge in [self.unique[best_p2][4][0], self.unique[best_p2][5][0]]
+                and len(self.unique[edge][3]) == 1  # node degree 2
             ):
                 self.unique[edge][7] = best_p2
             else:
@@ -341,7 +373,7 @@ class COINS:
 
         return edge_gdf
 
-    def _add_gdf_stroke_attributes(self):
+    def _add_gdf_stroke_attributes(self, return_ends=False):
         # Invert self.edge_idx to get a dictionary where the key is
         # the original edge index and the value is the group
         inv_edges = {
@@ -353,6 +385,16 @@ class COINS:
         for edge in self.uv_index:
             stroke_group_attributes.append(inv_edges[edge])
 
+        if return_ends:
+            ends_bool = {k: False for k in self.uv_index}
+            for vals in self.unique.values():
+                if isinstance(vals[6], str) or isinstance(vals[7], str):
+                    ends_bool[vals[8]] = True
+
+            return (
+                pd.Series(stroke_group_attributes, index=self.edge_gdf.index),
+                pd.Series(ends_bool.values(), index=self.edge_gdf.index),
+            )
         return pd.Series(stroke_group_attributes, index=self.edge_gdf.index)
 
 
@@ -374,118 +416,55 @@ def _list_to_pairs(in_list):
     return [list(pair) for pair in zip(tmp_list[:-1], tmp_list[1:], strict=True)]
 
 
-def _compute_angle(point1, point2):
-    """Calculates the angle between two points in space."""
-    height = abs(point2[1] - point1[1])
-    base = abs(point2[0] - point1[0])
-    angle = round(math.degrees(math.atan(height / base)), 3)
-    return angle
-
-
-def _compute_orientation(line):
-    """Calculates the orientation of a line segment. Point1 is
-    the lower one on the y-axes and vice versa for Point2.
+def _angle_between_two_lines(line1, line2):
+    """
+    Computes interior angle between 2 lines.
+        input:  line<x> ... list of 2 tuples (x,y);
+                line1 and line2 by definition share one unique tuple
+                (overlap in 1 point)
+        returns:    interior angle in degrees 0<alpha<=180
+                    (we assume that line1!=line2, so alpha=0 not possible)
     """
-    point1 = line[1]
-    point2 = line[0]
 
-    # If the latutide of a point is less and the longitude is more, or
-    # If the latitude of a point is more and the longitude is less, then
-    # the point is oriented leftward and wil have negative orientation.
-    if ((point2[0] > point1[0]) and (point2[1] < point1[1])) or (
-        (point2[0] < point1[0]) and (point2[1] > point1[1])
-    ):
-        return -_compute_angle(point1, point2)
+    # extract points
+    a, b = tuple(line1[0]), tuple(line1[1])
+    c, d = tuple(line2[0]), tuple(line2[1])
 
-    # if the latitudes are same, the line is horizontal
-    elif point2[1] == point1[1]:
+    # assertion: we expect exactly 2 of the 4 points to be identical
+    # (lines touch in this point)
+    points = collections.Counter([a, b, c, d])
+
+    # make sure lines are not identical
+    if len(points) == 2:
+        warnings.warn(
+            f"Lines are between points {points.keys()} identical. Please revise input "
+            "data to ensure no lines are identical or overlapping. "
+            "You can check for duplicates using `gdf.geometry.duplicated()`. Assuming"
+            "an angle of 0 degrees.",
+            UserWarning,
+            stacklevel=3,
+        )
         return 0
 
-    # if the longitudes are same, the line is vertical
-    elif point2[0] == point1[0]:
-        return 90
+    # make sure lines do touch
+    if len(points) == 4:
+        raise ValueError("Lines do not touch.")
 
-    return _compute_angle(point1, point2)
+    # points where line touch = "origin" (for vector-based angle calculation)
+    origin = [k for k, v in points.items() if v == 2][0]
+    # other 2 unique points (one on each line)
+    point1, point2 = (k for k, v in points.items() if v == 1)
 
+    # translate lines into vectors (numpy arrays)
+    v1 = [point1[0] - origin[0], point1[1] - origin[1]]
+    v2 = [point2[0] - origin[0], point2[1] - origin[1]]
 
-def _points_set_angle(line1, line2):
-    """Calculate the acute joining angle between two given set of points."""
-    l1orien = _compute_orientation(line1)
-    l2orien = _compute_orientation(line2)
-
-    if ((l1orien > 0) and (l2orien < 0)) or ((l1orien < 0) and (l2orien > 0)):
-        return abs(l1orien) + abs(l2orien)
-
-    elif ((l1orien > 0) and (l2orien > 0)) or ((l1orien < 0) and (l2orien < 0)):
-        theta1 = abs(l1orien) + 180 - abs(l2orien)
-        theta2 = abs(l2orien) + 180 - abs(l1orien)
-        if theta1 < theta2:
-            return theta1
-        else:
-            return theta2
-
-    elif (l1orien == 0) or (l2orien == 0):
-        if l1orien < 0:
-            return 180 - abs(l1orien)
-        elif l2orien < 0:
-            return 180 - abs(l2orien)
-        else:
-            return 180 - (
-                abs(_compute_orientation(line1)) + abs(_compute_orientation(line2))
-            )
-
-    elif l1orien == l2orien:
-        return 180
-
-
-def _angle_between_two_lines(line1, line2):
-    """Calculate the joining angle between two line segments."""
-    l1p1, l1p2 = line1
-    l2p1, l2p2 = line2
-    l1orien = _compute_orientation(line1)
-    l2orien = _compute_orientation(line2)
-
-    # If both lines have same orientation, return 180 If one of the
-    # lines is zero, exception for that If both the lines are on same side
-    # of the horizontal plane, calculate 180-(sumOfOrientation) If both the
-    # lines are on same side of the vertical plane, calculate pointSetAngle.
-    if l1orien == l2orien:
-        angle = 180
-
-    elif (l1orien == 0) or (l2orien == 0):
-        angle = _points_set_angle(line1, line2)
-
-    elif l1p1 == l2p1:
-        if ((l1p1[1] > l1p2[1]) and (l1p1[1] > l2p2[1])) or (
-            (l1p1[1] < l1p2[1]) and (l1p1[1] < l2p2[1])
-        ):
-            angle = 180 - (abs(l1orien) + abs(l2orien))
-        else:
-            angle = _points_set_angle([l1p1, l1p2], [l2p1, l2p2])
-
-    elif l1p1 == l2p2:
-        if ((l1p1[1] > l2p1[1]) and (l1p1[1] > l1p2[1])) or (
-            (l1p1[1] < l2p1[1]) and (l1p1[1] < l1p2[1])
-        ):
-            angle = 180 - (abs(l1orien) + abs(l2orien))
-        else:
-            angle = _points_set_angle([l1p1, l1p2], [l2p2, l2p1])
-
-    elif l1p2 == l2p1:
-        if ((l1p2[1] > l1p1[1]) and (l1p2[1] > l2p2[1])) or (
-            (l1p2[1] < l1p1[1]) and (l1p2[1] < l2p2[1])
-        ):
-            angle = 180 - (abs(l1orien) + abs(l2orien))
-        else:
-            angle = _points_set_angle([l1p2, l1p1], [l2p1, l2p2])
-
-    elif l1p2 == l2p2:
-        if ((l1p2[1] > l1p1[1]) and (l1p2[1] > l2p1[1])) or (
-            (l1p2[1] < l1p1[1]) and (l1p2[1] < l2p1[1])
-        ):
-            angle = 180 - (abs(l1orien) + abs(l2orien))
-        else:
-            angle = _points_set_angle([l1p2, l1p1], [l2p2, l2p1])
+    # compute angle between 2 vectors in degrees
+    dot_product = v1[0] * v2[0] + v1[1] * v2[1]
+    norm_v1 = math.sqrt(v1[0] ** 2 + v1[1] ** 2)
+    norm_v2 = math.sqrt(v2[0] ** 2 + v2[1] ** 2)
+    cos_theta = round(dot_product / (norm_v1 * norm_v2), 6)  # precision issues fix
+    angle = math.degrees(math.acos(cos_theta))
 
     return angle
 
