@@ -1,12 +1,13 @@
 #!/usr/bin/env python
-
 import math
+import os
 import warnings
 
 import geopandas as gpd
 import libpysal
 import networkx as nx
 import numpy as np
+import pandas as pd
 from numpy.lib import NumpyVersion
 from shapely.geometry import Point
 
@@ -18,6 +19,69 @@ __all__ = [
 ]
 
 
+def deprecated(new_way):
+    """
+    Decorator to mark classes as deprecated and point towards functional API.
+    """
+
+    def decorator(func1):
+        import functools
+
+        @functools.wraps(func1)
+        def new_func1(*args, **kwargs):
+            if os.getenv("ALLOW_LEGACY_MOMEPY", "False").lower() not in (
+                "true",
+                "1",
+                "yes",
+            ):
+                warnings.warn(
+                    f"Class based API like `momepy.{func1.__name__}` is deprecated. "
+                    f"Replace it with `momepy.{new_way}` to use functional API instead "
+                    "or pin momepy version <1.0. Class-based API will be removed in "
+                    "1.0. "
+                    # "See details at https://docs.momepy.org/en/stable/migration.html",
+                    "",
+                    FutureWarning,
+                    stacklevel=2,
+                )
+            return func1(*args, **kwargs)
+
+        return new_func1
+
+    return decorator
+
+
+def removed(new_way):
+    """
+    Decorator to mark classes as deprecated and removed from momepy.
+    """
+
+    def decorator(func1):
+        import functools
+
+        @functools.wraps(func1)
+        def new_func1(*args, **kwargs):
+            if os.getenv("ALLOW_LEGACY_MOMEPY", "False").lower() not in (
+                "true",
+                "1",
+                "yes",
+            ):
+                warnings.warn(
+                    f"`momepy.{func1.__name__}` is deprecated. Replace it with "
+                    f"{new_way} "
+                    "or pin momepy version <1.0. This class will be removed in 1.0. "
+                    # "See details at https://docs.momepy.org/en/stable/migration.html"
+                    "",
+                    FutureWarning,
+                    stacklevel=2,
+                )
+            return func1(*args, **kwargs)
+
+        return new_func1
+
+    return decorator
+
+
 def unique_id(objects):
     """
     Add an attribute with a unique ID to each row of a GeoDataFrame.
@@ -47,10 +111,15 @@ def _angle(a, b, c):
     return abs((a2 - a1 + 180) % 360 - 180)
 
 
-def _generate_primal(graph, gdf_network, fields, multigraph, oneway_column=None):
+def _generate_primal(
+    graph, gdf_network, fields, multigraph, oneway_column=None, preserve_index=False
+):
     """Generate a primal graph. Helper for ``gdf_to_nx``."""
     graph.graph["approach"] = "primal"
 
+    if gdf_network.index.name is not None:
+        graph.graph["index_name"] = gdf_network.index.name
+
     msg = (
         " This can lead to unexpected behaviour. "
         "The intended usage of the conversion function "
@@ -70,33 +139,48 @@ def _generate_primal(graph, gdf_network, fields, multigraph, oneway_column=None)
             category=RuntimeWarning,
             stacklevel=3,
         )
+    custom_index = not gdf_network.index.equals(pd.RangeIndex(len(gdf_network)))
 
-    key = 0
-    for row in gdf_network.itertuples():
+    for i, row in enumerate(gdf_network.itertuples()):
         first = row.geometry.coords[0]
         last = row.geometry.coords[-1]
 
         data = list(row)[1:]
         attributes = dict(zip(fields, data, strict=True))
+        if preserve_index:
+            attributes["index_position"] = i
+            if custom_index:
+                attributes["index"] = row.Index
         if multigraph:
-            graph.add_edge(first, last, key=key, **attributes)
-            key += 1
+            graph.add_edge(first, last, **attributes)
 
             if oneway_column:
                 oneway = bool(getattr(row, oneway_column))
                 if not oneway:
-                    graph.add_edge(last, first, key=key, **attributes)
-                    key += 1
+                    graph.add_edge(last, first, **attributes)
         else:
             graph.add_edge(first, last, **attributes)
 
+    node_attrs = {node: {"x": node[0], "y": node[1]} for node in graph.nodes}
+    nx.set_node_attributes(graph, node_attrs)
 
-def _generate_dual(graph, gdf_network, fields, angles, multigraph, angle):
+
+def _generate_dual(
+    graph, gdf_network, fields, angles, multigraph, angle, preserve_index
+):
     """Generate a dual graph. Helper for ``gdf_to_nx``."""
     graph.graph["approach"] = "dual"
+
+    if gdf_network.index.name is not None:
+        graph.graph["index_name"] = gdf_network.index.name
+
+    custom_index = not gdf_network.index.equals(pd.RangeIndex(len(gdf_network)))
+
     key = 0
 
-    sw = libpysal.weights.Queen.from_dataframe(gdf_network, silence_warnings=True)
+    sw = libpysal.weights.Queen.from_dataframe(
+        gdf_network, silence_warnings=True, use_index=False
+    )
     cent = gdf_network.geometry.centroid
     gdf_network["temp_x_coords"] = cent.x
     gdf_network["temp_y_coords"] = cent.y
@@ -105,6 +189,10 @@ def _generate_dual(graph, gdf_network, fields, angles, multigraph, angle):
         centroid = (row.temp_x_coords, row.temp_y_coords)
         data = list(row)[1:-2]
         attributes = dict(zip(fields, data, strict=True))
+        if preserve_index:
+            attributes["index_position"] = i
+            if custom_index:
+                attributes["index"] = row.Index
         graph.add_node(centroid, **attributes)
 
         if sw.cardinalities[i] > 0:
@@ -150,6 +238,8 @@ def gdf_to_nx(
     angles=True,
     angle="angle",
     oneway_column=None,
+    integer_labels=False,
+    preserve_index=False,
 ):
     """
     Convert a LineString GeoDataFrame to a ``networkx.MultiGraph`` or other
@@ -188,6 +278,16 @@ def gdf_to_nx(
         path traversal by specifying the boolean column in the GeoDataFrame. Note,
         that the reverse conversion ``nx_to_gdf(gdf_to_nx(gdf, directed=True,
         oneway_column="oneway"))`` will contain additional duplicated geometries.
+    integer_labels : bool, default False
+        Convert node labels to integers. By default, node labels are tuples with (x, y)
+        coordinates. Set to True to encode them as integers. Note that the x, and y
+        coordinates are always preserved as node attributes.
+    preserve_index : bool, default False
+        Preserve information about the index of ``gdf_network``. If
+        ``gdf_network.index`` is the default ``RangeIndex``, ``"index_position"``
+        attribute is added to each edge. If it is a custom index, ``"index_position"``
+        and ``"index"`` attributes are added. These attributes are then used by
+        :func:`nx_to_gdf` to faithfully roundtrip the data in the same order.
 
     Returns
     -------
@@ -258,14 +358,27 @@ def gdf_to_nx(
                 "Bidirectional lines are only supported for directed graphs."
             )
 
-        _generate_primal(net, gdf_network, fields, multigraph, oneway_column)
+        _generate_primal(
+            net,
+            gdf_network,
+            fields,
+            multigraph,
+            oneway_column,
+            preserve_index=preserve_index,
+        )
 
     elif approach == "dual":
         if directed:
             raise ValueError("Directed graphs are not supported in dual approach.")
 
         _generate_dual(
-            net, gdf_network, fields, angles=angles, multigraph=multigraph, angle=angle
+            net,
+            gdf_network,
+            fields,
+            angles=angles,
+            multigraph=multigraph,
+            angle=angle,
+            preserve_index=preserve_index,
         )
 
     else:
@@ -273,6 +386,9 @@ def gdf_to_nx(
             f"Approach '{approach}' is not supported. Use 'primal' or 'dual'."
         )
 
+    if integer_labels:
+        net = nx.convert_node_labels_to_integers(net)
+
     return net
 
 
@@ -300,6 +416,15 @@ def _lines_to_gdf(net, points, node_id):
 
     if "crs" in net.graph:
         gdf_edges.crs = net.graph["crs"]
+    if "index_position" in gdf_edges.columns:
+        gdf_edges = gdf_edges.sort_values("index_position").drop(
+            columns="index_position"
+        )
+    if "index" in gdf_edges.columns:
+        gdf_edges = gdf_edges.set_index("index")
+    else:
+        gdf_edges = gdf_edges.reset_index(drop=True)
+    gdf_edges.index.name = net.graph.get("index_name", None)
 
     return gdf_edges
 
@@ -331,12 +456,25 @@ def _dual_to_gdf(net):
     """Generate a linestring gdf from a dual network. Helper for ``nx_to_gdf``."""
     starts, edge_data = zip(*net.nodes(data=True), strict=True)
     gdf_edges = gpd.GeoDataFrame(list(edge_data))
+    if "index_position" in gdf_edges.columns:
+        gdf_edges = gdf_edges.sort_values("index_position").drop(
+            columns="index_position"
+        )
+    if "index" in gdf_edges.columns:
+        gdf_edges = gdf_edges.set_index("index")
+    else:
+        gdf_edges = gdf_edges.reset_index(drop=True)
+    gdf_edges.index.name = net.graph.get("index_name", None)
     gdf_edges.crs = net.graph["crs"]
     return gdf_edges
 
 
 def nx_to_gdf(
-    net, points=True, lines=True, spatial_weights=False, nodeID="nodeID"  # noqa
+    net,
+    points=True,
+    lines=True,
+    spatial_weights=False,
+    nodeID="nodeID",  # noqa: N803
 ):
     """
     Convert a ``networkx.Graph`` to a LineString GeoDataFrame and Point GeoDataFrame.
@@ -457,7 +595,6 @@ def limit_range(vals, rng):
         The limited array.
     """
 
-    vals = np.asarray(vals)
     nan_tracker = np.isnan(vals)
 
     if (len(vals) > 2) and (not nan_tracker.all()):
@@ -467,11 +604,9 @@ def limit_range(vals, rng):
             method = {"interpolation": "nearest"}
         rng = sorted(rng)
         if nan_tracker.any():
-            lower = np.nanpercentile(vals, rng[0], **method)
-            higher = np.nanpercentile(vals, rng[1], **method)
+            lower, higher = np.nanpercentile(vals, rng, **method)
         else:
-            lower = np.percentile(vals, rng[0], **method)
-            higher = np.percentile(vals, rng[1], **method)
+            lower, higher = np.percentile(vals, rng, **method)
         vals = vals[(lower <= vals) & (vals <= higher)]
 
     return vals
